#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#else
#include <sys/time.h>
#include <time.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include "ctype.h"
#include "math.h"
#include "SDL.h"
#include "SDL_ttf.h"

#include "auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

#define SINCOS_TABLE_SIZE	4096

/* Tablas precalculadas: */ 
bool sincos_table=false;
float sin_table[SINCOS_TABLE_SIZE];
float cos_table[SINCOS_TABLE_SIZE];
int rayo_prec_lx=0,rayo_prec_ly=0,rayo_prec_dx=0,rayo_prec_dy=0;
float *rayo_x_prec=0,*rayo_y_prec=0,*rayo_z_prec=0,*px_prec=0,*pz_prec=0;
int reflection_center_x=320;
int reflection_center_y=240;


void reflection_init(void)
{
	bool sincos_table=false;
	if (rayo_x_prec!=0) delete []rayo_x_prec;
	if (rayo_y_prec!=0) delete []rayo_y_prec;
	if (rayo_z_prec!=0) delete []rayo_z_prec;
	if (px_prec!=0) delete []px_prec;
	if (pz_prec!=0) delete []pz_prec;
	rayo_prec_lx=0;
	rayo_prec_ly=0;
	rayo_prec_dx=0;
	rayo_prec_dy=0;
	rayo_x_prec=0;
	rayo_y_prec=0;
	rayo_z_prec=0;
	px_prec=0;
	pz_prec=0;

} /* reflection_init */ 


void reflection_draw(SDL_Surface *orig,int y,SDL_Surface *reflection,float kh,float kv,float oh,float ov,float wave_height,int center_x,int center_y)
{
	int i,j;
	int ax,ay;
	float fax,fax_inc;
	float rayo_x,rayo_y,rayo_z;
	float px,pz;
	float ix,iy;
	float image_z=0;
	float image_x1=-10.0F,image_x2=10.0F,image_y1=-20.0F,image_y2=0.0F;
	float inv_image_x2_x1,inv_image_y2_y1;
	float wave_normal_y=-5.0F;
	float inv_wave_normal_y=1.0F/wave_normal_y;
	float reflection_y=0.0F;
	float h1,h2,h3;
	float nx,ny,nz;
	int ah,av,inc=SINCOS_TABLE_SIZE/64;
	float c,k;
	float rx,ry,rz;
	float inv_pi2=1.0F/(3.141592F*2.0F);
	float inv_destw=1.0F/float(orig->w);
	int projected_x;
	int projected_y;
	Uint32 *dest_ptr;
	int prec_offs;

	wave_normal_y=(-50.0F)/float(wave_height);
	inv_wave_normal_y=1.0F/wave_normal_y;

	if (!sincos_table) {
		for(i=0;i<SINCOS_TABLE_SIZE;i++) {
			sin_table[i]=float(sin((float(i)*(3.141592F*2.0F))/float(SINCOS_TABLE_SIZE)));
			cos_table[i]=float(cos((float(i)*(3.141592F*2.0F))/float(SINCOS_TABLE_SIZE)));
		} /* for */ 
		sincos_table=true;
	} /* if */ 


	// calcular las constantes para escalar la reflexin al tamao adecuado:
	{
		ax=int(( (float(SINCOS_TABLE_SIZE)/6.0F)*(-center_x) ) * inv_destw);
		ay=int(( (float(SINCOS_TABLE_SIZE)/6.0F)*(y-center_y) ) * inv_destw);
		ax=ax%SINCOS_TABLE_SIZE;
		ay=ay%SINCOS_TABLE_SIZE;
		if(ax<0) ax+=SINCOS_TABLE_SIZE;
		if(ay<0) ay+=SINCOS_TABLE_SIZE;
		rayo_x=sin_table[ax];
		rayo_y=cos_table[ax]*sin_table[ay];
		rayo_z=cos_table[ax]*cos_table[ay];
		reflection_y=float( 1 + sin( ( (3.141592F/3.0F)*( y-center_y) ) * inv_destw ));
		k=float(reflection_y / rayo_y);
		image_z=rayo_z*k-1;
		image_x1=rayo_x*k;				

		ax=int(( (float(SINCOS_TABLE_SIZE)/6.0F)*((orig->w-1)-center_x) ) * inv_destw);
		ay=int(( (float(SINCOS_TABLE_SIZE)/6.0F)*(y-center_y) ) * inv_destw);
		ax=ax%SINCOS_TABLE_SIZE;
		ay=ay%SINCOS_TABLE_SIZE;
		if(ax<0) ax+=SINCOS_TABLE_SIZE;
		if(ay<0) ay+=SINCOS_TABLE_SIZE;
		rayo_x=sin_table[ax];
		rayo_y=cos_table[ax]*sin_table[ay];
		rayo_z=cos_table[ax]*cos_table[ay];
		reflection_y=float( 1 + sin( ( (3.141592F/3.0F)*( y-center_y) ) * inv_destw ));
		k=float(reflection_y / rayo_y);
		image_x2=rayo_x*k;
		image_y1=-((image_x2-image_x1)*float(y))/float(2*orig->w);
		image_y2=0.0F;

		inv_image_x2_x1=1.0F/(image_x2-image_x1);
		inv_image_y2_y1=1.0F/(image_y2-image_y1);

		fax_inc=float(SINCOS_TABLE_SIZE)*0.16666667F * inv_destw;
	}


	if (rayo_prec_lx!=reflection->w ||
		rayo_prec_ly!=reflection->h ||
		rayo_prec_dx!=orig->w ||
		rayo_prec_dy!=orig->h ||
		reflection_center_x!=center_x ||
		reflection_center_y!=center_y) {
		rayo_prec_lx=reflection->w;
		rayo_prec_ly=reflection->h;
		rayo_prec_dx=orig->w;
		rayo_prec_dy=orig->h;
		reflection_center_x=center_x;
		reflection_center_y=center_y;
		if (rayo_x_prec!=0) delete []rayo_x_prec;
		if (rayo_y_prec!=0) delete []rayo_y_prec;
		if (rayo_z_prec!=0) delete []rayo_z_prec;
		if (px_prec!=0) delete []px_prec;
		if (pz_prec!=0) delete []pz_prec;

		rayo_x_prec=new float[rayo_prec_lx*rayo_prec_ly];
		rayo_y_prec=new float[rayo_prec_lx*rayo_prec_ly];
		rayo_z_prec=new float[rayo_prec_lx*rayo_prec_ly];
		px_prec=new float[rayo_prec_lx*rayo_prec_ly];
		pz_prec=new float[rayo_prec_lx*rayo_prec_ly];

		for(j=0;j<reflection->h;j++) {
			ay=int(( (float(SINCOS_TABLE_SIZE)*0.16666667F)*((y+j)-center_y) ) * inv_destw);
			ay=ay%SINCOS_TABLE_SIZE;
			if(ay<0) ay+=SINCOS_TABLE_SIZE;
			fax=( (float(SINCOS_TABLE_SIZE)*0.16666667F)*(-center_x) ) * inv_destw;

			for(i=0;i<reflection->w;i++) {
				// rayo:
				fax+=fax_inc;
				ax=int(fax);
				ax=ax%SINCOS_TABLE_SIZE;
				if(ax<0) ax+=SINCOS_TABLE_SIZE;
				rayo_x_prec[i+j*reflection->w]=sin_table[ax];
				rayo_y_prec[i+j*reflection->w]=cos_table[ax]*sin_table[ay];
				rayo_z_prec[i+j*reflection->w]=cos_table[ax]*cos_table[ay];

				// punto de colisin con lago:
				if (rayo_y_prec[i+j*reflection->w]!=0) {
					k=float( reflection_y / rayo_y_prec[i+j*reflection->w]);
					px_prec[i+j*reflection->w]=rayo_x_prec[i+j*reflection->w]*k;
					pz_prec[i+j*reflection->w]=rayo_z_prec[i+j*reflection->w]*k-1;
				} else {
					px_prec[i+j*reflection->w]=0;
					pz_prec[i+j*reflection->w]=0;
				} /* if */ 

			} /* for */ 
		} /* for */ 

	} /* if */ 


	prec_offs=0;
	for(j=0;j<reflection->h;j++) {
		dest_ptr=((Uint32* )((Uint8 *)reflection->pixels + j * reflection->pitch));

		for(i=0;i<reflection->w;i++) {
			rayo_x=rayo_x_prec[prec_offs];
			rayo_y=rayo_y_prec[prec_offs];
			rayo_z=rayo_z_prec[prec_offs];
			px=px_prec[prec_offs];
			pz=pz_prec[prec_offs++];

			// Clculo de la normal:
			// funcin: sin(kh*px+oh)*sin(kv*pz+ov)
			ah=int(kh*px+oh)&0x0fff;
			av=int(kv*pz+ov)&0x0fff;
			
			h1=sin_table[ah]*sin_table[av];
			h2=sin_table[(ah+inc)&0x0fff]*sin_table[av];
			h3=sin_table[ah]*sin_table[(av+inc)&0x0fff];
			nx=h2-h1;
			ny=wave_normal_y;
			nz=h3-h1;

//					n=1.0F/float(sqrt(nx*nx+ny*ny+nz*nz));
//					nx*=n; ny*=n; nz*=n;
			nx*=inv_wave_normal_y;
			ny*=inv_wave_normal_y;
			nz*=inv_wave_normal_y;

			// Clculo del vector reflejado:
			c=2*(nx*rayo_x+ny*rayo_y+nz*rayo_z);
			rx = c*nx - rayo_x;
			ry = c*ny - rayo_y;
			rz = c*nz - rayo_z;

			// clculo del punto de colisin con la imagen de fondo:
			if (rz!=0) {
				k=(image_z-pz)/rz;
				ix=px+k*rx;
				iy=k*ry;
			} else {
				ix=0;
				iy=0;
			} /* if */ 
			
			// calcular el color del pixel:
			projected_x=int(float(orig->w)*(ix-image_x1)*inv_image_x2_x1);
			projected_y=int(float(y)*(iy-image_y1)*inv_image_y2_y1);

			if (projected_x<0) projected_x=0;
			if (projected_x>=orig->w) projected_x=orig->w-1;
			if (projected_y<0) projected_y=0;
			if (projected_y>=y) projected_y=y-1;

			*dest_ptr=*((Uint32* )((Uint8 *)orig->pixels + projected_y * orig->pitch + (projected_x << 2)));
			dest_ptr++;
		} /* for */ 
	} /* for */ 


} /* reflection_draw */ 
