#ifdef _WIN32
#include "windows.h"
#include "direct.h"
#endif

#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_net.h"
#include "BList.h"
#include "auxiliar.h"

#include "conditionscript.h"
#include "actionscript.h"
#include "animscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "interfaces.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "mog2multiplayer.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "GamePreviewDlg.h"
#include "NetworkGameWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X;
extern int SCREEN_Y;
extern int COLOUR_DEPTH;
extern bool fullscreen;
extern int extra_flags;
extern char *application_name;

/* Redrawing constant: */ 
int NETWORKGAME_REDRAWING_PERIOD=40;
extern SDL_Surface *screen_sfc;

extern int buffer_fps,act_fps,fps_timer;
extern int buffer_fps_d,fps_d;


NetworkGameWindow::NetworkGameWindow(int dx,int dy,int gm,char *file,NWMOG2GameInfo *g,TTF_Font *font) : BActiveWindow(0,0,dx,dy,font)
{
	game=new ActiveNetworkMOG2(g,gm,file);

} /* NetworkGameWindow::NetworkGameWindow */ 


NetworkGameWindow::~NetworkGameWindow()
{
	delete game;
} /* NetworkGameWindow:: */ 


void NetworkGameWindow::Execute(void)
{
	int time,act_time;
	SDL_Event event;
	bool quit=false;
	bool waiting=false;

	active_window=this;

	SDL_QuitSubSystem(SDL_INIT_VIDEO);
	SDL_InitSubSystem(SDL_INIT_VIDEO);
	screen_sfc = SDL_SetVideoMode(old_dx, old_dy, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0)|extra_flags);
	SDL_WM_SetCaption(application_name, 0);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableUNICODE(1);

	time=SDL_GetTicks();
	while (!quit) {
		while( SDL_PollEvent( &event ) ) {
			bool captured=false;

            switch( event.type ) {
                /* Keyboard event */ 
                case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_F12 ||
						event.key.keysym.sym==SDLK_ESCAPE) {
						captured=true;
						quit = true;
					} /* if */ 

					if (event.key.keysym.sym==SDLK_RETURN) {
						SDLMod modifiers=SDL_GetModState();

						if ((modifiers&KMOD_ALT)!=0) {
							captured=true;

							/* Toogle FULLSCREEN mode: */ 
							if (fullscreen) fullscreen=false;
									   else fullscreen=true;
							SDL_QuitSubSystem(SDL_INIT_VIDEO);
							SDL_InitSubSystem(SDL_INIT_VIDEO);
							if (SDL_WasInit(SDL_INIT_VIDEO)) {
								screen_sfc = SDL_SetVideoMode(old_dx, old_dy, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0)|extra_flags);
								SDL_WM_SetCaption(application_name, 0);
								SDL_ShowCursor(SDL_DISABLE);
								SDL_EnableUNICODE(1);
							} else {
								quit = true;
							} /* if */ 
						} /* if */ 
					} /* if */ 

					if (event.key.keysym.sym==SDLK_w) {
						SDLMod modifiers=SDL_GetModState();

						if ((modifiers&KMOD_CTRL)!=0) {
							captured=true;
							waiting=true;
						} /* if */ 
					} /* if */ 
                    break;

				case SDL_KEYUP:
					if (event.key.keysym.sym==SDLK_w) {
						SDLMod modifiers=SDL_GetModState();

						if ((modifiers&KMOD_CTRL)!=0) {
							captured=true;
							waiting=false;
						} /* if */ 
					} /* if */ 
					break;

                /* SDL_QUIT event (window close) */ 
                case SDL_QUIT:
					captured=true;

                    quit = true;
                    break;
            } /* switch */ 

			if (!captured) {
				if (!this->event(&event)) quit=true;
			} /* if */ 
        } /* while */ 

		act_time=SDL_GetTicks();
		if (act_time-time>=NETWORKGAME_REDRAWING_PERIOD)
		{
			act_fps++;
			if ((act_time-fps_timer)>1000) {
				fps_timer=act_time;
				buffer_fps=act_fps;
				buffer_fps_d=fps_d;
				act_fps=0;
				fps_d=0;
			} /* if */ 

			time+=NETWORKGAME_REDRAWING_PERIOD;
			if ((act_time-time)>8*NETWORKGAME_REDRAWING_PERIOD) time=act_time;

			check_mouse();
			dirty=true;
			if (!waiting && !cycle(old_x,old_y)) {
				quit=true;
			} /* if */ 
		}

		if (!game->listening_cycle()) quit=true;

		if (!quit) {
			fps_d++;
			SDL_FillRect(screen_sfc,0,0);
			if (game!=0) {
				game->game->draw(screen_sfc);
//				game->game->sound(0,-1,-1);
			} /* if */ 

			/* show INFO: */ 
			{
				char tmp[256];
				List<ActiveOBJECT> l;
				ActiveOBJECT *ao;
				int y=0;

				sprintf(tmp,"FPS: %i/%i    CYCLE: %i/%i",buffer_fps,buffer_fps_d,
					    game->current_cycle,game->last_synchronized_cycle);
				print_left(tmp,font,255,255,255,screen_sfc,0,0);

				{
					/* Show Network connection status: */ 
					int i;
					int r,g;
					int delay=MAX_HYPOTHESES-((game->current_cycle-game->last_synchronized_cycle)&0xff);

					draw_rectangle(screen_sfc,old_dx-40,1,MAX_HYPOTHESES*4+2,10,SDL_MapRGB(screen_sfc->format,255,255,255));
					for(i=0;i<delay;i++) {
						r=400-i*64;
						if (r<0) r=0;
						if (r>255) r=255;
						g=400-(MAX_HYPOTHESES-i)*64;
						if (g<0) g=0;
						if (g>255) g=255;
						draw_line(screen_sfc,old_dx-37+i*4,3,old_dx-37+i*4,8,SDL_MapRGB(screen_sfc->format,r,g,0));
						draw_line(screen_sfc,old_dx-36+i*4,3,old_dx-36+i*4,8,SDL_MapRGB(screen_sfc->format,r,g,0));
					} /* for */ 
				}

				sprintf(tmp,"ACTIVE MAPS: [ ");
				{
					List<ActiveMAP> l2;
					ActiveMAP *map;
					char tmp2[80];

					l2.Instance(game->game->active_maps);
					l2.Rewind();
					while(l2.Iterate(map)) {
						sprintf(tmp2,"%s ",map->map->name);
						strcat(tmp,tmp2);
					} /* while */ 
					strcat(tmp,"]");
				}
				print_left(tmp,font,255,255,255,screen_sfc,0,10);

				y=20;
				{
					List<ActiveMAP> l4;
					ActiveMAP *map;

					l4.Instance(game->game->active_maps);
					l4.Rewind();
					while(l4.Iterate(map)) {
						sprintf(tmp,"MAP R.: %i  -  INACT R.: %i  -  ACT R.: %i ",map->map->rooms.Length(),map->inactive_rooms.Length(),map->active_rooms.Length());
						{
							List<ActiveROOM> l2;
							List<ROOM> l3;
							ROOM *rtmp;
							ActiveROOM *r;
							char tmp2[80];
							int i,j;

							l2.Instance(map->active_rooms);
							l2.Rewind();
							while(l2.Iterate(r)) {
								l3.Instance(map->map->rooms);
								l3.Rewind();
								i=-1;
								j=0;
								while(l3.Iterate(rtmp)) {
									j++;
									if (rtmp==r->room) i=j;
								} /* while */ 
								sprintf(tmp2,"%i[ %i,%i ]{%i,%i} ",i,r->objects[0].Length(),r->objects[1].Length(),
																   r->nonpassive_objects[0].Length(),r->nonpassive_objects[1].Length());
								strcat(tmp,tmp2);
							} /* while */ 
						}
						print_left(tmp,font,255,255,255,screen_sfc,0,y);
						y+=10;
					} /* while */ 
				}

				l.Instance(game->game->player_objects);
				l.Rewind();
				while(l.Iterate(ao)) {
					int ox,oy;
					int x1,y1,x2,y2;
					ao->get_bbox(&x1,&y1,&x2,&y2);
					ao->hotspot_coordinates(&ox,&oy);
					sprintf(tmp,"CHARACTER, X,Y: %i,%i,%i,%i  -  ENERGY: %i  -  STATE: %i (%i)  -  INSTANCES:  %i ",
						x1,y1,x2,y2,
						ao->energy,ao->state,ao->state_timmer,
						ao->other_instances.Length()+1);

					{
						List<ActiveOBJECT> l2;
						ActiveOBJECT *o;
						char tmp2[80];

						ao->absolute_hotspot_coordinates(&ox,&oy);
						sprintf(tmp2,"[%i,%i] ",ox,oy);
						strcat(tmp,tmp2);

						l2.Instance(ao->other_instances);
						l2.Rewind();
						while(l2.Iterate(o)) {
							o->absolute_hotspot_coordinates(&ox,&oy);
							sprintf(tmp2,"[%i,%i] ",ox,oy);
							strcat(tmp,tmp2);
						} /* while */ 
					}

					print_left(tmp,font,255,255,255,screen_sfc,0,y);
					y+=10;
				} /* if */ 
			}
			SDL_Flip(screen_sfc);
		} /* if */ 

	}

	SDL_QuitSubSystem(SDL_INIT_VIDEO);
	SDL_InitSubSystem(SDL_INIT_VIDEO);
	screen_sfc = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0));
	SDL_WM_SetCaption(application_name, 0);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableUNICODE(1);

} /* NetworkGameWindow::Execute */ 


bool NetworkGameWindow::cycle(int stx,int sty)
{
	return game->execute_cycle();
} /* NetworkGameWindow::cycle */ 



void NetworkGameWindow::contens_size(int *dx,int *dy)
{
} /* NetworkGameWindow::contens_size */ 


void NetworkGameWindow::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
} /* NetworkGameWindow::draw_contens */ 


bool NetworkGameWindow::event(SDL_Event *event)
{
	if (active_window!=this) return true;

	game->event(event);

	return true;
} /* NetworkGameWindow::Event */ 
