#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "POVariableWnd.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X,SCREEN_Y;

POVariableWnd::POVariableWnd(int x,int y,int dx,int dy,PLACED_OBJECT *o,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	pobject=o;
	if (pobject!=0 && pobject->object!=0) pobject->object->get_variable_names(&variable_names);
} /* POVariableWnd::OVariableWnd */ 


POVariableWnd::~POVariableWnd()
{
} /* POVariableWnd::~POVariableWnd */ 


bool POVariableWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);


	/* Object variables: */ 
	if (( (mouse_button&SDL_BUTTON(1) &&
		  !(old_mouse_button&SDL_BUTTON(1)) ) ||
		  (mouse_button&SDL_BUTTON(3) &&
		  !(old_mouse_button&SDL_BUTTON(3)) ) )) {

		List<char> l;
		char *vn;
		List<VARIABLE> l2;
		VARIABLE *v2;
		bool found;
		char *tmp;
		int y;
		int end_x;

		y=3;

		l.Instance(variable_names);
		l.Rewind();
		while(l.Iterate(vn)) {
			found=false;

			l2.Instance(pobject->modified_variables);
			l2.Rewind();
			while(!found && l2.Iterate(v2)) {
				if (strcmp(vn,v2->name)==0) found=true;
			} /* while */ 

			if (found) {
				tmp=v2->generate_string();
				end_x=get_text_width(tmp,font)+get_text_width(v2->name,font)+get_text_width(": ",font)+3;

				delete []tmp;
			} else {
				int iv;
				char *sv,tmp2[256];
				int value_type;

				value_type=pobject->object->get_variable_value(vn,&iv,&sv);
				if (value_type==VALUE_INTEGER) {
					sprintf(tmp2,"%s: %i",vn,iv);
				} else {
					if (value_type==VALUE_STRING) {
						sprintf(tmp2,"%s: %s",vn,sv);
					} else {
						sprintf(tmp2,"%s: -",vn);
					} /* if */ 
				} /* if */ 			
				end_x=3+get_text_width(tmp2,font);
			} /* if */ 

			if (local_mouse_x>=3 && local_mouse_x<end_x &&
				local_mouse_y>=y && local_mouse_y<y+TTF_FontHeight(font)) {
				BVChoiceDlg *dlg;
				int res=0;

				dlg=new BVChoiceDlg(-1,-1,0,00,"Choose action:","Reset\nInteger Value\nString Value\n",&res,font);
				dlg->Execute();
				delete dlg;
				
				if (res==1) {
					/* Reset variable: */ 
					List<VARIABLE> l;
					VARIABLE *v2;
					bool found;

					l2.Instance(pobject->modified_variables);
					l2.Rewind();
					found=false;
					while(!found && l2.Iterate(v2)) {
						if (v2->cmp(vn)) {
							found=true;
							pobject->modified_variables.DeleteElement(v2);
							delete v2;
						} /* if */ 
					} /* while */ 
					set_dirty();
				} /* if */ 

				if (res==2 || res==3) {
					/* Integer Value: */ 
					int y,x;
					bool found;
					char tmp2[256];

					l2.Instance(pobject->modified_variables);
					l2.Rewind();
					found=false;
					while(!found && l2.Iterate(v2)) 
						if (v2->cmp(vn)) found=true;

					if (!found) {
						v2=new VARIABLE(vn,0);
						pobject->modified_variables.Add(v2);
					} /* if */ 

					if (res==2) {
						BIntegerEdDlg *dlg;
					
						if (v2->type!=VALUE_INTEGER) v2->ivalue=0;
						v2->type=VALUE_INTEGER;
						new_window_coordinates(140,60,&x,&y);
						dlg=new BIntegerEdDlg(x,y,140,60,"Enter initial value:",&v2->ivalue,8,font);
						dlg->Execute();
						delete dlg;
					} else {
						BStringEdDlg *dlg;

						if (v2->type==VALUE_STRING) strcpy(tmp2,v2->svalue);
											  else tmp2[0]=0;
						v2->type=VALUE_STRING;
						new_window_coordinates(int(SCREEN_X*0.8),60,&x,&y);
						dlg=new BStringEdDlg(x,y,int(SCREEN_X*0.8),60,"Enter initial value:",tmp2,80,"",font);
						dlg->Execute();
						delete dlg;
						v2->set(tmp2);
					} /* if */ 

					set_dirty();
				} /* if */ 
			} /* if */ 

			y+=10;
		} /* while */ 
	} /* if */ 

	return true;
} /* POVariableWnd::cycle */ 


void POVariableWnd::contens_size(int *dx,int *dy)
{
	List<VARIABLE> l2;
	VARIABLE *v2;
	List<char> l;
	char *vn;
	bool found;
	char *tmp;
	int y;
	int len;

	BWindow::contens_size(dx,dy);

	y=3;

	l.Instance(variable_names);
	l2.Instance(pobject->modified_variables);
	l.Rewind();
	while(l.Iterate(vn)) {
		found=false;

		l2.Rewind();
		while(!found && l2.Iterate(v2)) {
			if (strcmp(vn,v2->name)==0) found=true;
		} /* while */ 

		if (found) {
			tmp=v2->generate_string();
			len=get_text_width(tmp,font)+get_text_width(v2->name,font)+get_text_width(": ",font)+3;
			if (len>*dx) *dx=len;

			delete []tmp;
		} else {
			int iv;
			char *sv,tmp2[256];
			int value_type;

			value_type=pobject->object->get_variable_value(vn,&iv,&sv);
			if (value_type==VALUE_INTEGER) {
				sprintf(tmp2,"%s: %i",vn,iv);
			} else {
				if (value_type==VALUE_STRING) {
					sprintf(tmp2,"%s: %s",vn,sv);
				} else {
					sprintf(tmp2,"%s: -",vn);
				} /* if */ 
			} /* if */ 			
			len=3+get_text_width(tmp2,font);
			if (len>*dx) *dx=len;
		} /* if */ 

		y+=10;
	} /* while */ 
	if (y>*dy) *dy=y;
} /* POVariableWnd::contens_size */ 


void POVariableWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	List<VARIABLE> l2;
	VARIABLE *v2;
	List<char> l;
	char *vn;
	bool found;
	char *tmp,*tmp2;
	int y;

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	y=3;

	l.Instance(variable_names);
	l2.Instance(pobject->modified_variables);
	l.Rewind();
	while(l.Iterate(vn)) {
		found=false;
		
		l2.Rewind();
		while(!found && l2.Iterate(v2)) {
			if (strcmp(vn,v2->name)==0) found=true;
		} /* while */ 

		if (found) {
			tmp=v2->generate_string();
			tmp2=new char[strlen(tmp)+strlen(v2->name)+3];
			sprintf(tmp2,"%s: %s",v2->name,tmp);

			print_left(tmp2,font,255,255,255,sfc,3-stx,y-sty);

			delete []tmp;
			delete []tmp2;
		} else {
			int iv;
			char *sv,tmp3[256];
			int value_type;

			value_type=pobject->object->get_variable_value(vn,&iv,&sv);
			if (value_type==VALUE_INTEGER) {
				sprintf(tmp3,"%s: %i",vn,iv);
			} else {
				if (value_type==VALUE_STRING) {
					sprintf(tmp3,"%s: %s",vn,sv);
				} else {
					sprintf(tmp3,"%s: -",vn);
				} /* if */ 
			} /* if */ 			

			print_left(tmp3,font,255,255,255,sfc,3-stx,y-sty);
		} /* if */ 

		y+=10;
	} /* while */ 

} /* POVariableWnd::draw_contens */ 

